@@ -2,7 +2,7 @@
  * Function.java
  *
  * Copyright (C) 2002-2005 Peter Graves
- * $Id: Function.java 13445 2011-08-06 14:46:28Z ehuelsmann $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -44,7 +44,7 @@ public abstract class Function extends Operator
      * The value of *load-truename* which was current when this function
      * was loaded, used for fetching the class bytes in case of disassembly.
      */
-    private final LispObject loadedFrom;
+    public final LispObject loadedFrom;
 
     protected Function() {
 	LispObject loadTruename = Symbol.LOAD_TRUENAME.symbolValueNoThrow();
