@@ -2,7 +2,7 @@
  * Java.java
  *
  * Copyright (C) 2002-2006 Peter Graves, Andras Simon
- * $Id: Java.java 14694 2014-04-25 07:55:53Z mevenson $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -228,10 +228,7 @@ public final class Java
         catch (IllegalArgumentException e) {
             error(new LispError("illegal argument"));
         }
-        catch (Throwable t) { // no code -> no ControlTransfer
-            error(new LispError(getMessage(t)));
-        }
-        // Not reached.
+        // Not reached
         return NIL;
     }
 
@@ -659,7 +656,7 @@ public final class Java
 
     private static final Primitive JARRAY_REF = new pf_jarray_ref();
     @DocString(name="jarray-ref", args="java-array &rest indices",
-    doc="Dereferences the Java array JAVA-ARRAY using the given INDICIES, " +
+    doc="Dereferences the Java array JAVA-ARRAY using the given INDICES, " +
         "coercing the result into a Lisp object, if possible.")
     private static final class pf_jarray_ref extends Primitive
     {
@@ -677,7 +674,7 @@ public final class Java
 
     private static final Primitive JARRAY_REF_RAW = new pf_jarray_ref_raw();
     @DocString(name="jarray-ref-raw", args="java-array &rest indices",
-    doc="Dereference the Java array JAVA-ARRAY using the given INDICIES. " +
+    doc="Dereference the Java array JAVA-ARRAY using the given INDICES. " +
         "Does not attempt to coerce the result into a Lisp object.")
     private static final class pf_jarray_ref_raw extends Primitive
     {
@@ -695,7 +692,7 @@ public final class Java
 
     private static final Primitive JARRAY_SET = new pf_jarray_set();
     @DocString(name="jarray-set", args="java-array new-value &rest indices",
-    doc="Stores NEW-VALUE at the given index in JAVA-ARRAY.")
+    doc="Stores NEW-VALUE at the given INDICES in JAVA-ARRAY.")
     private static final class pf_jarray_set extends Primitive
     {
         pf_jarray_set()
