@@ -62,21 +62,26 @@
      ,@forms))
 
 (defun compute-restarts (&optional condition)
+  (let ((res ()))
+    (map-restarts (lambda(restart) (push restart res)) condition t)
+    (nreverse res)))
+
+(defun map-restarts (fn condition call-test-p)
   (let ((associated ())
 	(other ()))
     (dolist (alist *condition-restarts*)
       (if (eq (car alist) condition)
 	  (setq associated (cdr alist))
 	  (setq other (append (cdr alist) other))))
-    (let ((res ()))
-      (dolist (restart-cluster *restart-clusters*)
-        (dolist (restart restart-cluster)
-          (when (and (or (not condition)
-                         (member restart associated)
-                         (not (member restart other)))
-                     (funcall (restart-test-function restart) condition))
-            (push restart res))))
-      (nreverse res))))
+    (dolist (restart-cluster *restart-clusters*)
+      (dolist (restart restart-cluster)
+	(when (and (or (not condition)
+		       (member restart associated)
+		       (not (member restart other)))
+		   (or (not call-test-p)
+		       (funcall (restart-test-function restart) condition)))
+	  (funcall fn restart))))))
+
 
 (defun restart-report (restart stream)
   (funcall (or (restart-report-function restart)
@@ -105,7 +110,16 @@
 	     :format-arguments (list identifier))))
 
 (defun invoke-restart (restart &rest values)
-  (let ((real-restart (find-restart-or-control-error restart)))
+  (let ((real-restart
+	  (if (restart-p restart)
+	      (catch 'found
+		(map-restarts (lambda(r) (when (eq r restart)
+					   (throw 'found r)))
+			      nil nil)
+		(error 'control-error
+		       :format-control "Restart ~S is not active."
+		       :format-arguments (list restart)))
+	      (find-restart-or-control-error restart))))
     (apply (restart-function real-restart) values)))
 
 (defun interactive-restart-arguments (real-restart)
@@ -115,11 +129,20 @@
         '())))
 
 (defun invoke-restart-interactively (restart)
-  (let* ((real-restart (find-restart-or-control-error restart))
-         (args (interactive-restart-arguments real-restart)))
+  (let* ((real-restart
+	   (if (restart-p restart)
+	       (catch 'found
+		 (map-restarts (lambda(r) (when (eq r restart)
+					    (throw 'found r)))
+			       nil nil)
+		 (error 'control-error
+			:format-control "Restart ~S is not active."
+			:format-arguments (list restart)))
+	       (find-restart-or-control-error restart)))
+	 (args (interactive-restart-arguments real-restart))
+	 )
     (apply (restart-function real-restart) args)))
 
-
 (defun parse-keyword-pairs (list keys)
   (do ((l list (cddr l))
        (k '() (list* (cadr l) (car l) k)))
